/// <reference types="node" />
import { IPublicKey } from './interfaces';
/**
 * Represents a public key for operations with {@link VirgilCrypto}.
 *
 * `VirgilPublicKey` objects are not meant to be created directly using the `new` keyword.
 * Use the {@link VirgilCrypto.generateKeys} and {@link VirgilCrypto.importPublicKey} methods
 * to create `VirgilPublicKey` instances.
 *
 * @protected
 */
export declare class VirgilPublicKey implements IPublicKey {
    /**
     * Public key identifier. Note that the public key and its
     * corresponding private key will have the same identifier.
     * */
    identifier: Buffer;
    /**
     * The public key material. Unlike the private keys, the public
     * key material is available as a property of the `PublicKey` object.
     */
    key: Buffer;
    constructor(identifier: Buffer, key: Buffer);
}
