/// <reference types="node" />
import { VirgilPrivateKey } from "./VirgilPrivateKey";
/**
 * Gets the private key bytes of the given private key object from internal store.
 * @param {VirgilPrivateKey} privateKey - Private key object.
 * @returns {Buffer} - Private key bytes.
 *
 * @hidden
 */
export declare function getPrivateKeyBytes(privateKey: VirgilPrivateKey): Buffer;
/**
 * Saves the private key bytes corresponding to the given private key object into
 * internal buffer.
 *
 * @param {VirgilPrivateKey} privateKey - Private key object.
 * @param {Buffer} bytes - Private key bytes.
 *
 * @hidden
 */
export declare function setPrivateKeyBytes(privateKey: VirgilPrivateKey, bytes: Buffer): void;
/**
 * Checks if the private key bytes corresponding to the given private key
 * object exist in the internal buffer.
 *
 * @hidden
 *
 * @param { VirgilPrivateKey } privateKey - Private key object.
 *
 * @returns {boolean}
 */
export declare function hasPrivateKeyBytes(privateKey: VirgilPrivateKey): boolean;
